/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package comgr;

import java.awt.*;
import java.awt.image.*;

/**
 * An image filter that "disables" an image by turning
 * it into a grayscale image, and brightening the pixels
 * in the image.
 */
public class GrayFilter extends RGBImageFilter 
{
    private boolean brighter;
    private int percent;
    
    
    /**
     * Constructs a GrayFilter object that filters a color image to a 
     * grayscale image. Used by buttons to create disabled ("grayed out")
     * button images.
     *
     * @param b  a boolean -- true if the pixels should be brightened
     * @param p  an int in the range 0..100 that determines the percentage
     *           of gray, where 100 is the darkest gray, and 0 is the lightest
     */
    public GrayFilter(boolean b, int p) 
    {
        brighter = b;
        percent = p;

	// canFilterIndexColorModel indicates whether or not it is acceptable
	// to apply the color filtering of the filterRGB method to the color
	// table entries of an IndexColorModel object in lieu of pixel by pixel
     	// filtering.
        canFilterIndexColorModel = true;
    	
		//{{INIT_CONTROLS
		//}}
}
    
    /**
     * Overrides <code>RGBImageFilter.filterRGB</code>.
     */
    public int filterRGB(int x, int y, int rgb) 
    {
        // Use NTSC conversion formula.
	int gray = (int)
	          (
	            (
	              ((rgb >> 16) & 0xff) * 3 / 10 + 
                  ((rgb >> 8) & 0xff) * 59 / 100 + 
                  (rgb & 0xff) * 11 / 100
                ) / 3
              );
	
        if (brighter) 
        {
            gray = (255 - ((255 - gray) * (100 - percent) / 100));
        } else 
        {
            gray = (gray * (100 - percent) / 100);
        }
	
        if (gray < 0) gray = 0;
        if (gray > 255) gray = 255;
        return (rgb & 0xff000000) | (gray << 16) | (gray << 8) | (gray << 0);
    }
    
    
    /** Creates a disabled image   */
    public static Image createDisabledImage (Image i) 
    {
	 GrayFilter filter = new GrayFilter(true, 50);
	 ImageProducer prod = new FilteredImageSource(i.getSource(), filter);
	 Image grayImage = Toolkit.getDefaultToolkit().createImage(prod);
	 return grayImage;
    }
    
	
	//{{DECLARE_CONTROLS
	//}}
};

