/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.util.Vector;


/**
 * CPanel is a Panel extension which provides for
 * a alternate panels.
 * When extending from CPanel be sure to super()
 * during construction and to super.handleEvent(evt) from
 * handleEvent if you override it.
 * <p>
 * setIndex():     to add a new panel to the panel.
 * select():     to show (activate) a panel
 */
public class CPanel extends Panel
{
    Vector vPanels;

    // Zero-relative index of currently shown panel. -1 if no tabs exist or are shown.
    protected int curIndex = -1;


     private Component userPanel = null;

     transient private int osAdjustment;

     private int lastWidth = -1;


    /**
     * Constructs a CPanel
     */
     public CPanel()
     {
        super();
      	super.setLayout(null);
	vPanels = new Vector();
     }


   public void setBackground(Color b)
   {
     super.setBackground(b);
   }


   public void setCurrent(int index)
   {
        if(index == curIndex)	return;
        curIndex = index;
    }


 /**
   * Adds a new panel
   * @return the zero-relative index of the newly added panel
   */
    public int setIndex()
    {
      int index;

      index = vPanels.size() - 1;

      if (curIndex == -1)
      {
         setCurrent(index);
      }
      return (index);
  }


   /**
     * Adds a new panel and associated panel.
     * The panel is added at the specified index.
     * @param panel the panel to associate with the panel
     */
   public int addPanel(Component panel)
    {
       vPanels.addElement(panel);
       return setIndex();
    }


   /**
     * Adds a component to the end of this container.
     * It is overridden so that it only appends to the TabPanel.
     *
     * @param comp the component to add
     * @return the added component
     */

   public synchronized Component add(Component comp)
    {
      int newIndex = addPanel(comp);
      if (newIndex == curIndex) select(newIndex);
      repaint();
      return comp;
   }


/**
 * Gets the index for a specific panel.
 * @param panel the panel to get the index of
 */
   public synchronized int getPanelIndex(Component panel)
   {
	return vPanels.indexOf(panel);
   }



/**
 * Selects the panel at the given index, showing it
 * and its associated panel.
 * The panel is activated, ready for user input.
 * @param index zero-relative index of the panel to select
 */

   public void setPanel(int index)
   {
     if(index == curIndex) return;
     select(index);
   }

   public void first() { select(0); }
   public void last() {  select(vPanels.size() - 1); }

   public synchronized void select(int index)
   {
      if ((index < 0) || (index >= vPanels.size())) return;

      Component p = (Component) vPanels.elementAt(index);
      setCurrent(index);
      showPanel(p);
  }


    /**
     * Adds the panel to the base panel and shows it.
     * Hides all other (previous) panels instead of removing them.
     * @param p the Panel to add and show
     */
    public void showPanel(Component p)
    {
	if (userPanel != null)	userPanel.setVisible(false);

	userPanel = p;
	if (userPanel != null)
	{
		Component[] comps = getComponents();
		int l = comps.length;
		int x;
		// recherche le panel en argument
		for (x = 0; x < l; x++)
		{
                  if (comps[x] == userPanel)  break;
		}
		if (x == l) super.add(userPanel, -1);
                curIndex = x;

		userPanel.setVisible(true);
		userPanel.requestFocus();
                validate();
		repaint();
	    }
    }


/**
 * Gets the panel for the panel at the given index.
 * @param index zero-relative index of the panel
 * @return returns the Panel associated with the panel
 */

   public synchronized Component getTabPanel(int index)
   {
	if ((index < 0) || (index >= vPanels.size())) return null;
	Component p = (Component)vPanels.elementAt(index);
	return p;
   }



   /**
     * Handles the laying out of components within this component.
     * This is a standard Java AWT method which gets called by the AWT
     * when this component is validated with the validate() method.
     *
     * @see java.awt.Container#validate
     */


   public void doLayout()
   {
	Rectangle r = getBounds();
	if (userPanel != null)
	{
          userPanel.setBounds(0, 0, r.width, r.height);
          userPanel.invalidate();
	  userPanel.validate();
   //       if (userPanel instanceof Canvas || userPanel instanceof Panel)
          {
	    userPanel.repaint();
	  }
//	  else
	  {
	    repaint();
          }
	}
   }



	/**
	 * Takes no action.
	 * Standard Java AWT method
	 * Since layout managers CANNOT BE USED with this container the standard
	 * setLayout has been OVERRIDDEN for this container and does nothing.
	 **/
    public void setLayout(LayoutManager mgr) {}



	/**
	 * Tells this component that it has been added to a container.
	 * Standard Java AWT method.
	 * It has been overridden here to hook-up event listeners.
	 *
	 * @see #removeNotify
	 */
	public synchronized void addNotify()
	{
		super.addNotify();
	}

	/**
	 * Tells this component that it is being removed from a container.
	 * Standard Java AWT method.
	 * It has been overridden here to unhook event listeners.
	 */
	public synchronized void removeNotify()
	{
		super.removeNotify();
	}



    public Dimension getPreferredSize()
    {
    	Dimension p = getSize();
    	Dimension m = getMinimumSize();
    	return new Dimension(Math.max(p.width, m.width), Math.max(p.height, m.height));
    }

    public Dimension getMinimumSize()
    {
        return new Dimension(20, 40);
    }



}

