/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import common.*;

/**
   Defines a selected/unselected Checkbox class.
   Listeners can register to listen for ItemChanged Events on the Checkbox.
 */

public class DCheckbox extends Component implements ItemSelectable
{
   protected String label;
   protected boolean state = false;
   protected ItemListener itemListener;

   static final boolean CLEARED = false;
   static final boolean CHECKED = true;

   static final int LEFT = 0;
   static final int RIGHT = 1;
   public int position;    // Position du carr

   static final int BEVEL_NONE = 0;
   static final int BEVEL_LINE = 1;
   static final int BEVEL_RAISED = 2;
   static final int BEVEL_LOWERED = 3;
   public int border;

   int BORDER_X = 0;
   int BORDER_Y = 0;


   int checksize = 16;

   /**
     * Constructs the DCheckbox with an initial state of unchecked.
    */

    public DCheckbox() { this("", false ); }

   /**
     * Constructs the DCheckbox by calling the other DCheckbox constructor
     * with the label argument and an initial state of unchecked (false).
    */

    public DCheckbox(String label ) { this(label, false ); }

    /**
     * Constructs the DCheckbox, setting its label to the label argument
     * and the state of cheked or unchecked to the state argument.
     */
    public DCheckbox(String label, boolean state )
    {
        super();
        enableEvents( java.awt.AWTEvent.MOUSE_EVENT_MASK ); // enable processEvent

        this.label = label;
        this.state = state;
        border = BEVEL_NONE;
        position = LEFT;
    }

    /**
     * Overridden to call repaint
     */
    public void setEnabled( boolean state )
    {
        super.setEnabled( state );
        repaint();
    }

    public void setBevel(int b) { border = b; }

    /**
       Overridden to return either the user-established size
         set by calling "setSize",
       or the size as determined by calling "AdjustSize".
       Must be called after component has had a graphics
       context established for it.

       @return Dimension representing the component's preferred size
     */

    public java.awt.Dimension getPreferredSize()
    {
     Dimension actsize = getSize();
     Dimension bestsize = adjustSize();

     int aw = actsize.width;
     int ah = actsize.height;
     int bw = bestsize.width;
     int bh = bestsize.height;

     if((bw > aw) || (bh > ah)) return bestsize;

     return actsize;
    }

    /** Overridden to return the preferred size of the component  */
    public java.awt.Dimension getMinimumSize() { return getPreferredSize();}


   /**
     * Sets the label of the DCheckbox to the newLabel argument.  Calls
     * <code>invalidate</code> to force the layout the DCheckbox again.
    */

    public synchronized void setLabel( String newLabel )
    {
        label = newLabel;
        repaint();
    }


    /** Returns the String label of the DCheckbox */

    public String getLabel() { return label; }


   /** Sets the state of the DCheckbox to the newState argument */

    public void setState( boolean newState )
    {
        state = newState;
        repaint();
    }

   /** Returns the boolean state of the DCheckbox */

    public boolean getState() { return state; }

    public void setBorder(int b) { border = b; }

   /**
    * Gets the optimum size for this DCheckbox, which will be calculated to encompass
    * any label and checkable box, with a few pixels extra on all four sides.
   */


   protected java.awt.Dimension adjustSize()
   {
      int width  = checksize + BORDER_X * 2;
      int height = checksize + BORDER_Y * 2;;
      int lwidth = 0;
      int lheight = 0;

      Graphics g = this.getGraphics();
      // Calculate actual dimensions
      if( g != null )
      {
         java.awt.FontMetrics fm = g.getFontMetrics();

         // Calculate title dimensions
         if( label != null )
         {
          lheight = fm.getHeight();    // Get more accurate height
          if(lheight > height)
          {
            BORDER_Y = lheight - height;
            BORDER_Y = (BORDER_Y + 1) / 2;
          }
          else if(checksize > lheight)
           lheight = checksize;

          lwidth = 8 + fm.stringWidth(label) + BORDER_X * 2;   // Calculate label width
         }
      }

      if(lheight > height) height = lheight;
      return new java.awt.Dimension( width + lwidth, height );
   }

   /**
      Cette mthode est appele par la fonction "add" du container
   */

    public void addNotify()
    {
       super.addNotify();
       setSize(getPreferredSize() );
    }

    /**
     * Overridden to call paint without clearing the
     * screen so as to reduce flicker.
     */
    public void update( java.awt.Graphics g ) { paint( g ); }



    /** Paints the DCheckbox  */

    public void paint( java.awt.Graphics g )
    {
       int w = getWidth();
       int h = getHeight();
       Color background = getBackground();
       Color light = background.brighter();
       Color dark = background.darker();


       // Dessine le fond
       if(border > 0)
       {
        g.setColor(background);
        g.fillRect(0, 0, w, h);
       }

       // Dessine le bord
       if(border == BEVEL_LINE)
       {
         g.setColor(getForeground());
         g.drawRect(0, 0, w - 1, h - 1);
       }
       else if(border == BEVEL_RAISED)
       {
        g.setColor(light);
        g.drawLine( 0, 0, w - 1, 0 );   // Bord clair
        g.drawLine( 0, 1, 0, h - 1 );

        g.setColor(dark);
        g.drawLine( 1, h - 1, w - 1, h - 1 );
        g.drawLine( 2, h - 2, w - 2, h - 2 );
        g.drawLine( w - 1, 1, w - 1, h - 1 );
       }
       else if(border == BEVEL_LOWERED)
       {
        g.setColor(dark);
        g.drawLine( 0, 0, w - 1, 0 );   // Bord clair
        g.drawLine( 0, 1, 0, h - 1 );

        g.setColor(light);
        g.drawLine( 1, h - 1, w - 1, h - 1 );
        g.drawLine( 2, h - 2, w - 2, h - 2 );
        g.drawLine( w - 1, 1, w - 1, h - 1 );
       }

       // Offet du carr
       int coffset;
       int loffset;

       if(position == LEFT)
       {
        coffset = BORDER_X;
        loffset = checksize + BORDER_X * 3 + 6;
       }
       else
       {
        coffset = w - (checksize + BORDER_X * 2) - 1;
        loffset = 6;
       }

       // Dessine Check square
       int xr = coffset + checksize - 1;
       int yb = BORDER_Y + checksize - 1;

       g.setColor( dark );
       g.drawLine( coffset, BORDER_Y, xr, BORDER_Y);
       g.drawLine( coffset, BORDER_Y, coffset, yb);
       g.drawLine( coffset + 1, BORDER_Y + 1, xr - 1, BORDER_Y + 1);
       g.drawLine( coffset + 1, BORDER_Y + 2, coffset + 1, yb - 1);

       g.setColor( light);
       g.drawLine( xr, BORDER_Y , xr, yb);
       g.drawLine( coffset, yb, xr, yb);
       g.setColor( dark );
       g.drawLine( xr - 1, BORDER_Y + 1, xr - 1, yb - 1);
       g.drawLine( coffset + 1, yb - 1, xr - 1, yb - 1);

       // Fond
       g.setColor( light );
       g.fillRect( coffset + 2, BORDER_Y + 2, checksize - 4, checksize - 4);

       // Dessine la croix
       if(state)
        {
          int cx = coffset + 4;
          int cy = BORDER_Y + 4;
          int cx1 = cx + checksize - 10;
          int cy1 = cy + checksize - 9;

          g.setColor(Color.black);
          g.drawLine(cx, cy, cx1, cy1);
          g.drawLine(cx, cy1, cx1, cy);
          g.drawLine(cx + 1, cy, cx1 + 1, cy1);
          g.drawLine(cx + 1, cy1, cx1 + 1, cy);
        }

       // Draw label
       if(label != null)
        if(label.length() > 0 )
        {
         FontMetrics fm = g.getFontMetrics();

         if(position == RIGHT)
           loffset = coffset - fm.stringWidth(label);

         int y = BORDER_Y + 14;

         if( isEnabled() ) g.setColor( Color.black );
         else              g.setColor( Color.gray );

         g.drawString( label, loffset, y );
       }

   } // paint



  /**
      process mouse clicks to change the Checkbox state.
   */
   public void processMouseEvent(MouseEvent evt )
   {
     if(evt.getID() == MouseEvent.MOUSE_PRESSED)
     if( isEnabled() )
      {
        state = !state;
        if( itemListener != null )
         {
          int stateChange = state ? ItemEvent.SELECTED : ItemEvent.DESELECTED;
          itemListener.itemStateChanged( new java.awt.event.ItemEvent (
                this, java.awt.event.ItemEvent.ITEM_STATE_CHANGED, this.label, stateChange ) );
         }
        repaint();
      }
   }


   /** Part of the ItemListener interface, returns the current Checkbox  */

    public Object[] getSelectedObjects()
    {
        Object[] objs = { this };
        return objs;
    }

    /**
     * Adds the specified item listener to recieve item events from
     */
    public synchronized void addItemListener(ItemListener l)
    {
       itemListener = AWTEventMulticaster.add(itemListener, l);
    }

    /**
     * Removes the specified item listener so that it no longer
     * receives item events from this checkbox.
     */
    public synchronized void removeItemListener(ItemListener l)
    {
        itemListener = AWTEventMulticaster.remove(itemListener, l);
    }


}
