/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.Dimension;
import java.awt.Point;
import java.awt.Component;
import java.awt.Adjustable;

/**
 * @deprecated
 * Use java.awt.ScrollPane instead.
 * @see java.awt.ScrollPane
 * The ScrollingPanel is a panel with automatic scroll bars.
 * It may contain only one component/container/panel/etc.
 * It automatically tracks the size of this (usually large) component and
 * provides scroll bars so the entire component may be viewed within a
 * panel of limited size.
 * Typically, the added component would be a panel (or some other container)
 * which would then contain a variety of other components.
 */
public class ScrollingPanel extends java.awt.ScrollPane
{
    /**
     * Constructs a default ScrollingPanel.
     * The panel is initialized with a null component, zero
     * minimum height and zero minimum width.
     */
    public ScrollingPanel()
    {
        super();
    }

    /**
     * Constructs a new ScrollingPanel initialized with the
     * specified scrollbarDisplayPolicy.
     * @param scrollbarDisplayPolicy policy for when scrollbars should be shown
     */
    public ScrollingPanel(int scrollbarDisplayPolicy)
    {
        super(scrollbarDisplayPolicy);
    }

    /**
     * Constructs a new ScrollingPanel initialized with the
     * specified component, minimum height and minimum width.
     * @param component the component (usually a Panel) to be
     * scrolled
     * @param minWidth the value to be used for the minimumSize()
     * width of the ScrollingPanel
     * @param minHeight the value to be used for the minimumSize()
     * height of the ScrollingPanel
     */
    public ScrollingPanel(Component component, int minWidth, int minHeight)
    {
        super();
        addImpl(component, null, -1);
        setSize(minWidth, minHeight);
    }

    /**
     * Sets the value to be used for the minimumSize() width
     * of the ScrollingPanel.
     * @param minWidth the value to be used for the minimumSize()
     * width of the ScrollingPanel
     * @see #getMinimumWidth
     */
    public void setMinimumWidth(int minWidth)
    {
        this.width = minWidth;
    }

    /**
     * Gets the current value used for the minimumSize()
     * width of the ScrollingPanel.
     * @return the current minimum width value
     */
    public int getMinimumWidth()
    {
        return this.width;
    }

    /**
     * Sets the value to be used for the minimumSize()
     * height of the ScrollingPanel.
     * @param minHeight the value to be used for the minimumSize()
     * height of the ScrollingPanel
     * @see #getMinimumHeight
     */
    public void setMinimumHeight(int minHeight)
    {
        this.height = minHeight;
    }

    /**
     * Gets the value used for the minimumSize() height
     * of the ScrollingPanel.
     * @return current minimum height value
     * @see #setMinimumHeight
     */
    public int getMinimumHeight()
    {
        return this.height;
    }


    /**
     * Sets the component in this ScrollingPanel. This is the component
     * that gets scrolled. The ScrollingPanel can only contain one component.
     * Any previous component will be removed before the new one is added.
     * @param comp the component to add
     */
    public void setComponent(Component comp)
    {
        addImpl(comp, null, -1);
    }

    /**
     * Gets the current component in the ScrollingPanel.
     * This is the component that gets scrolled.
     * @return the current component in the ScrollingPanel
     */
    public Component getComponent()
    {
	    if (getComponentCount() > 0) {
		    return getComponent(0);
	    }
	    else
            return null;
    }

    /**
     * Scrolls up by the number of pixels specified in the method setScrollLineIncrement().
     * The default is one pixel.
     */
    public void scrollUp()
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x, p.y - getVAdjustable().getUnitIncrement()));
    }

    /**
     * Scrolls left by the number of pixels specified in the method setScrollLineIncrement().
     * The default is one pixel.
     */
    public void scrollLeft()
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x - getHAdjustable().getUnitIncrement(), p.y));
    }

    /**
     * Scrolls down by the number of pixels specified in the method setScrollLineIncrement().
     * The default is one pixel.
     */
    public void scrollDown()
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x, p.y + getVAdjustable().getUnitIncrement()));
    }

    /**
     * Scrolls right by the number of pixels specified in the method setScrollLineIncrement().
     * The default is one pixel.
     */
    public void scrollRight()
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x + getHAdjustable().getUnitIncrement(), p.y));
    }

    /**
     * Scrolls one "page" up.
     * The page size is the size of this ScrollingPanel not including the
     * scroll bar, if present.
     */
    public void scrollPageUp()
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x, p.y - getVAdjustable().getBlockIncrement()));
    }

    /**
     * Scrolls one "page" left.
     * The page size is the size of this ScrollingPanel not including the
     * scroll bar, if present.
     */
    public void scrollPageLeft()
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x - getHAdjustable().getBlockIncrement(), p.y));
    }

    /**
     * Scrolls one "page" down.
     * The page size is the size of this ScrollingPanel not including the
     * scroll bar, if present.
     */
    public void scrollPageDown()
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x, p.y + getVAdjustable().getBlockIncrement()));
    }

    /**
     * Scrolls one "page" right.
     * The page size is the size of this ScrollingPanel not including the
     * scroll bar, if present.
     */
    public void scrollPageRight()
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x + getHAdjustable().getBlockIncrement(), p.y));
    }

    /**
     * Scrolls to an absolute vertical position.
     * The contained component's given pixel position will be scrolled to the
     * top line of the scrolling panel. The provided position is constrained to the range
     * [0, component's height - view port height].
     * @param position the pixel position to scroll to
     * @see #scrollHorizontalAbsolute
     */
    public void scrollVerticalAbsolute(int position)
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(p.x, position));
    }

    /**
     * Scrolls to an absolute horizontal position.
     * The contained component's given pixel position will be scrolled to the
     * leftmost part of the scrolling panel. The provided position is constrained to the range
     * [0, component's width - view port width].
     * @param position the pixel position to scroll to
     * @see #scrollVerticalAbsolute
     */
    public void scrollHorizontalAbsolute(int position)
    {
        Point p = getScrollPosition();
        setScrollPosition(new Point(position, p.y));
    }

    /**
	 * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
     *
     * For each axis, it returns the larger of the current size or the
     * minimum size.
     *
     * @see #minimumSize
	 */

    public Dimension getPreferredSize()
    {
        Dimension s = size();
        Dimension m = minimumSize();
        return new Dimension(Math.max(s.width, m.width), Math.max(s.height, m.height));
    }

    /**
	 * Returns the minimum dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the minimum size of this component.
     *
     * The value returned is set using the setMinimumHeight() and
     * setMinimumWidth() methods or when this ScrollingPanel is constructed.
     *
     * @see #preferredSize
     * @see #setMinimumHeight
     * @see #setMinimumWidth
	 */

    public Dimension getMinimumSize()
    {
        return new Dimension(width, height);

    }

    private int width = 100;
    private int	height = 100;

}

