/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;
import common.*;

/**
 * Components based on this class are used to select one value from 
 * a continuous range of values.
 * It has a movable thumb in front of a gauge with ticks marks on it.
 * <p>
 */
public abstract class Slider extends Canvas
{
    /** Defines the slider tick style */
    public static final int TICK_NONE       = 0;
    public static final int TICK_LEFT       = 1;
    public static final int TICK_RIGHT      = 2;
    public static final int TICK_BOTTOM     = TICK_LEFT;
    public static final int TICK_TOP        = TICK_RIGHT;
    public static final int TICK_BOTH       = 3;

    public static final int BORDER_NONE = 0;    
    public static final int BORDER_LOWERED = 1;    
    public static final int BORDER_RAISED  = 2;        

    /** Constructs the Slider */
    protected Slider()
    {
        this.style	= TICK_NONE;
        this.borderStyle = BORDER_NONE;
        this.min	= 0;
        this.max	= 255;
        this.freq	= 1;
        prevPos		=
        curPos		= 0;
        tick		= null;
    }



    /** Sets the current slider tick mark style  */
    public void setTickStyle(int style)
    {
        this.style = style;
        forceCallDoReshape = true;
    }

    /** Returns the current slider tick mark style  */
    public int getTickStyle()  {  return style;  }

    public void setBorderStyle(int s) 
    {
      switch( s )
      { 
       case BORDER_NONE:
       case BORDER_LOWERED: 
       case BORDER_RAISED:  borderStyle = s; break;
       default: break;
      }
    }
    

    /** Sets the minimum value of the slider  */
    public void setMinValue(int min)
    {
        if(min < 0) min = 0;
        this.min = min;
        forceCallDoReshape = true;
        repaint();
    }

    /** Returns the current minimum value of the slider */
    public int getMinValue() { return min; }

    /** Sets the maximum value of the slider  */
    public void setMaxValue(int max)
    {
      if(max < 0) max = 0;  
	  this.max = max;
	  forceCallDoReshape = true;
	  repaint();
    }

    /** Returns the current maximum value of the slider */
    public int getMaxValue() { return max; }

    /**
     * Sets the tick mark display frequency.
     * This is the range in value between each tick mark.
     */
    public void setTickFreq(int freq)
    {
     	if (this.freq != freq)
    	{
    		if(freq > max) freq = max;
    		if(freq < min) freq = min;

	        int pos = curPos * this.freq + min;
	        this.freq = freq;
	        doMove((pos - min) / this.freq, false);
			forceCallDoReshape = true;
	        repaint();
    	}
    }

    /** Returns the current tick mark display frequency */
    public int getTickFreq() { return freq; }


    /** Sets the starting slider value  Doesn't call repaint */

    /** Sets the slider value  */
    public void setValue(int pos)
    {
    	if (pos < getMinValue())    	pos = getMinValue();
    	else if (pos > getMaxValue())  	pos = getMaxValue();

        doMove((pos - min) / freq, false);
		forceCallDoReshape = true;
        repaint();
    }

    /** Returns the current slider value */
    public int getValue() { return curPos * freq + min; }

	/** Sets the command name of the action event fired by this component */
	public void setActionCommand(String command) { actionCommand = command; }
    public String getActionCommand() { return actionCommand; }

	/**
	  This abstract function is called by reshape.  
	  Override it to change the way the Slider is reshaped.
	 */
	protected abstract void do_reshape(int w, int h);

	/**
     * This routine updates the thumb position, paints the Slider, and
     * posts a new action event as needed. If the thumb position has
     * changed or the forcePost parameter is true the component will be painted
     * and an action event posted.
     * @param pos the new thumb position
     * @param forcePost true forces painting the slider and posting of an action
     * event even if the thumb position hasn't changed
	 */
    protected void doMove(int pos, boolean forcePost)
    {
        if (tick == null)  { prevPos = curPos = pos; return; }

        if (pos >= tick.length)  pos = tick.length - 1;

        if (pos != curPos || forcePost)
        {
			prevPos = curPos;
			curPos  = pos;

			Graphics g = getGraphics();
			if (g != null)
			{
			 paint(g);
			 g.dispose();
			} 
			g = null;

			sourceActionEvent();
        }
    }

    // Paint   Appelle seulement do_reshape si la taille a chang
    //         ou autre requte

    public void update(Graphics g) { paint(g); }

    /**
       Cette surcharge appelle seulement do_reshape
       do_mode est appelle par HSlider
       et donc appelle paint surcharge dans HSlider?
      */

    public void paint(Graphics g)
    {
       boolean callDoReshape = false;

       // If size of component changes, we want to call do_reshape
       Dimension currentSize = getSize();
       if(width != currentSize.width)
	   	{ 
          width = currentSize.width;
    	  callDoReshape = true;
    	}

       if(height != currentSize.height)
	   	{ 
          height = currentSize.height;
    	  callDoReshape = true;
    	}

       if(forceCallDoReshape || callDoReshape)
    	{
		  do_reshape(width, height);
		  forceCallDoReshape = false;
    	}
    }

    /**
     * Handles the mouse pressing or dragging this component's thumb.
     * It is not typically called directly.
     * @param i the mouse horizontal or vertical position coordinate
     * @param forcePost true forces painting the slider and posting of an action
     * event even if the thumb position hasn't changed
     */
    protected  void moveThumb(int i, boolean forcePost)
    {
        if(tick.length > 1)
        {
            int dist = tick[1].x - tick[0].x;
            if (dist == 0)  return;

            int newPos = (i - tick[0].x) / dist;
            if (newPos < 0) newPos = 0;

            if (((i - tick[0].x) % dist) > (dist / 2))  ++newPos;

            doMove(newPos, forcePost);
        }
    }

    /**
     * Adds the specified action listener to receive action events
     * from this component.
     */
	public void addActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this component.
     * @param l the action listener
     */
	public void removeActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}


    /** Fire an action event to the listeners */
	protected void sourceActionEvent()
	{
		if (actionListener != null)
			actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, actionCommand));
	}

    Color background = Color.lightGray; 
 
    public void setBackground(Color b)
    {
      background = b;
    } 
    
    public Color getBackground() { return background; }

    public void addNotify()
    {
     super.addNotify();   
     notified = true;
    }    

    protected int width;     /** The current component width */
    protected int height;    /** The current component height */
    /** Set if paint override should call DoReshape, even if the size has not changed  */
    protected boolean forceCallDoReshape = false;

    protected int style;   /** The current slider tick mark style */
    protected int freq;    /** The current tick mark display frequency */
    protected int min;     /** The minimum value of the slider range */
    protected int max;     /** The maximum value of the slider range */

    protected int prevPos;  /** The position of the thumb last paint */
    protected int curPos;   /** The current position of the thumb */

    /** The action listener to keep track of listeners for our action event */
	protected ActionListener		actionListener	= null;
	/** The command name of the action event fired by this component */
	protected String actionCommand = "Slider Moved";

	transient protected SliderTick tick[];	/** The tick marks for this slider */

 	protected static final int BORDER_X       = 12; // Left/Right margin
    protected static final int BORDER_Y       = 8; // Top/Bottom margin

    public int borderStyle;
    protected int tickWidth = 1;   // Largeur + espace entre tick
    protected int tickStep = 5;    // Un tick affich tous les ....
    
    String slidername;
    
    boolean notified = false;
}
