/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.*;
import java.awt.event.*;


/**
 * This component provides the up/down buttons used in spinner.
 */
public class SpinButtonPanel extends Panel implements ActionListener
{
    /**
     * Constructs a spinner with up/down buttons.
     */
    public SpinButtonPanel()
    {
    	incButton = new DirButton(DirButton.UP);
    	decButton = new DirButton(DirButton.DOWN);

    	incButton.setBevelHeight(1);
    	incButton.setShowFocus(false);
    	decButton.setBevelHeight(1);
    	decButton.setShowFocus(false);
//    	incButton.setArrowIndent(1);
//    	decButton.setArrowIndent(1);

        add(incButton);
        add(decButton);

       	setDelay(250);
       	setNotifyWhilePressed(true);

       	incButton.addActionListener(this);
       	decButton.addActionListener(this);
    }

    /**
     * Sets whether the spinner buttons will continually post notify events
     * while pressed.
     */
    public void setNotifyWhilePressed(boolean f)
    {
        incButton.setNotifyWhilePressed(f);
        decButton.setNotifyWhilePressed(f);
    }

    /**
     * Gets the current notifyWhilePressed status.
     */
    public boolean isNotifyWhilePressed()
    {
        return incButton.isNotifyWhilePressed();
    }

    /**
     * Sets the notification event delay of the spinner buttons in milliseconds.
     * @param d the delay between notification events in milliseconds
     */
    public void setDelay(int d)
    {
        incButton.setNotifyDelay(d);
        decButton.setNotifyDelay(d);
    }

    /**
     * Returns the current delay between notification events of the spinner
     * buttons in milliseconds.
     */
    public int getDelay()
    {
        return incButton.getNotifyDelay();
    }

	/**
	 * Takes no action.
	 * Since layout managers CANNOT BE USED with this container the standard
	 * setLayout has been OVERRIDDEN for this container and does nothing.
	 */
    public void setLayout(LayoutManager l)
    {
    }

    /**
     * Enables or disables this component so that it will respond to user input
     * or not.
     * This is a standard Java AWT method which gets called to enable or disable
     * this component. Once enabled this component will respond to user input.
     */
    public synchronized void setEnabled(boolean flag)
    {
      if(isEnabled() != flag)
       {
    	super.setEnabled(flag);
        incButton.setEnabled(flag);
        decButton.setEnabled(flag);
	   }
    }


    /**
     * This enables or disables the incrementing button only.
     */
    public synchronized void setUpButtonEnabled(boolean flag)
    {
    	incButton.setEnabled(flag);
    }

    /**
     * The enabled state of the incrementing button.
     */
    public boolean isUpButtonEnabled()
    {
    	return incButton.isEnabled();
    }

    /**
     * This enables or disables the decrementing button only.
     */
    public synchronized void setDownButtonEnabled(boolean flag)
    {
     	decButton.setEnabled(flag);
    }

    /**
     * The enabled state of the decrementing button.
     */
    public boolean isDownButtonEnabled()
    {
    	return decButton.isEnabled();
    }




    /**
     * Adds the specified action listener to receive action events.
     * The ActionCommand will be either "Increment" or "Decrement"
     * depending on which spinner button was pressed.
     * @param l the action listener
     */
	public void addActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this component.
     * @param l the action listener
     */
	public void removeActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}


    /**
     * Moves and/or resizes this component.
     * This is a standard Java AWT method which gets called to move and/or
     * resize this component. Components that are in containers with layout
     * managers should not call this method, but rely on the layout manager
     * instead.
     *
     * This method is overridden to reshape the two direction buttons.
     *
     * @param x horizontal position in the parent's coordinate space
     * @param y vertical position in the parent's coordinate space
     * @param width the new width
     * @param height the new height
     */
    public void setBounds(int x, int y, int width, int height)
    {
    	int halfHeight = height / 2;
    	int calcWidth = (int)(halfHeight * widthHeightRatio);

        incButton.setBounds(0, 0, calcWidth, halfHeight);
        decButton.setBounds(0, halfHeight, calcWidth, halfHeight);

        super.setBounds(x, y, width, height);
    }

	/**
	 * Returns the recommended dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the recommended size of this component.
	 */
    public Dimension getPreferredSize()
    {
    	int height = getSize().height;

		return new Dimension((int)((height / 2) * widthHeightRatio), height);
    }

    /**
	 * Returns the minimum dimensions to properly display this component.
     * This is a standard Java AWT method which gets called to determine
     * the minimum size of this component.
     * It simply returns the results of a call to preferedSize().
	 */
    public Dimension getMinimumSize()
    {
        return getPreferredSize();
    }

    /**
     * The ratio of width to height of the spinner buttons.  i.e. width is <ratio> * height.
     */
    protected double widthHeightRatio = 1.25;

	/**
	 * This is the Adjustment Event handling innerclass.
	 */
     public void actionPerformed(ActionEvent e)
      {
    	Object source = e.getSource();

    	if(source == incButton )	  sourceActionEvent( "Increment" );
    	else if(source == decButton ) sourceActionEvent( "Decrement");
      }

    /**
     * Fire an action event to the listeners
     */
	protected void sourceActionEvent(String actionCommand)
	{
		if (actionListener != null)
			actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, actionCommand));
	}


    /**
     * The button used to increment the spinner value.
     */
    protected DirButton incButton;
    /**
     * The button used to decrement the spinner value.
     */
    protected DirButton decButton;

    /**
     * The action listener to keep track of listeners for our action event.
     */
    protected ActionListener actionListener = null;


}
