/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.AWTEvent;
import java.awt.AWTEventMulticaster;
import common.*;

/**
 * Sets a timer to wait before an action event is posted.
 * The timer is implemented as a thread.  The one of the start(...) methods should
 * be called to start the thread.
 */

public class Timer implements Runnable
{
    /**
     * Creates a timer with the default delay.
     * After 1000 miliseconds this timer will fire an ActionEvent.
	 * It will not repeat.
     */
    public Timer() { this(1000, false); 		//{{INIT_CONTROLS
		//}}
}

    /**
     * Creates a timer with specified delay.
     * After the specified delay this timer will fire an ActionEvent.
	 * It will not repeat.
     * @param d the delay in milliseconds
     */
    public Timer(int d) { this(d, false);  }

    /**
     * Creates a timer with specified repeat setting and the default delay.
     * After 1000 miliseconds this timer will fire an ActionEvent.
	 * It may repeat, depending on r.
     * @param r if true, reset and repeat after generating the event
     */
    public Timer(boolean r)  { this(1000, r); }

    /**
     * Creates a timer with specified delay and repeat setting.
     * After the specified delay this timer will fire an ActionEvent.
	 * It may repeat, depending on r.
     * @param d the delay in milliseconds
     * @param r if true, reset and repeat after generating the event
     */
    public Timer(int d, boolean r)
    {
        delay     = d;
        repeat    = r;
        execute   = false;
        thread    = new Thread(this);
    }

    /**
     * Sets the delay time for this timer.
     * @param d the delay in milliseconds.  This delay will be used starting
     *          after the current delay elapses
     * @see #getDelay()
     */
    public void setDelay(int d) { delay = d; }

    /**
     * Obtains the delay time setting for this timer.
     */
    public int getDelay() { return delay; }


    /**
     * Changes the repeat setting of the timer.
     * If the repeat setting is false a single event will be generated.  When
     * set to true the timer produces a series of events.
     * @param f reset and repeat after generating the event
     * @see #isRepeat
     */
    public void setRepeat(boolean f) { repeat = f; }

    /**
     * Obtains the repeat setting of the timer.
     */
    public boolean isRepeat() { return repeat; }


    /**
     * Pauses the timer.
     */
    public void pause() { execute = false; }


    /**
     * Starts the timer with existing settings.
     */
    public void start()
    {
	    execute = true;
	    live	= true;
	    if(thread.isAlive()) return; // resume before
	    else
		{
	    	thread	= new Thread(this);
		    thread.start();
		}
    }

    /**
     * Starts the timer using the specified delay.
     */
    public void start(int d)
    {
	    setDelay(d);
	    start();
    }

    /**
     * Starts the timer using the specified repeat setting.
     */
    public void start(boolean r)
    {
	    setRepeat(r);
	    start();
    }

    /**
     * Starts the timer using the specified delay and repeat settings.
     */
    public void start(int d, boolean r)
    {
	    setDelay(d);
	    setRepeat(r);
	    start();
    }

    /**
     * Stops the timer.  After return the timer will generate no more events.
     */
    public void stop()
    {
	    execute		= false;
	    repeating	= false;
	    live		= false;
	    thread.interrupt();
    }

    /**
     * The thread body.  This method is called by the Java virtual machine
     * in response to a start call by the user.
     */
    public void run()
    {
        if(!execute) thread.interrupt();
		try
		{
            while(live)
            {
             do
             {
				repeating = repeat;
                thread.sleep(delay);
                if (execute) sourceActionEvent();
             }
             while (repeating && live);

             if((!execute && live) || !repeating) thread.interrupt();
            }
	   }
        catch (InterruptedException e) { }
    }

    /**
     * Sets the command name of the action event fired by this button.
     * @param command the name of the action event command fired by this button
     */
    public void setActionCommand(String command)
    {
        actionCommand = command;
    }

    /**
     * Returns the command name of the action event fired by this button.
     */
    public String getActionCommand()
    {
        return actionCommand;
    }

    /**
     * Adds the specified action listener to receive action events
     * from this button.
     */
	public void addActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.add(actionListener, l);
	}

    /**
     * Removes the specified action listener so it no longer receives
     * action events from this button.
     */
	public void removeActionListener(ActionListener l)
	{
		actionListener = AWTEventMulticaster.remove(actionListener, l);
	}

    /**
     * Fires an action event to the listeners.
     */
	public void sourceActionEvent()
	{
		if (actionListener != null)
			actionListener.actionPerformed(new ActionEvent(this, ActionEvent.ACTION_PERFORMED, actionCommand));
	}


    static public void Delay(int millis)
    {
      Thread  thread    = new Thread();

      thread.start();
      try { thread.sleep(millis);  }
      catch(InterruptedException e) { }
      thread.destroy();

    }



    protected int        eventType;
   	protected boolean    repeat;
    protected boolean    repeating;
    protected boolean    execute;
    protected boolean	   live;
    protected int        delay;
    protected String actionCommand;
	protected ActionListener actionListener = null;
    transient protected Thread thread;
}
