/*
    Advanced Explorer (c) 1999-2006 by D.G. Sureau

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    webmaster@scriptol.org
    http://www.scriptol.org
*/
package common;

import java.util.Vector;


public class TreeStructure extends DTreeView
{
    /**
     * Initializes the DTreeView from a string array.
     * There is one string for each node in the array. That string
     * contains the text of the node indented with same number of
     * leading spaces as the depth of that node in the tree.
     * @param s the string array used for initialization.
     * If null, the tree will be cleared.
     * @see #getTreeStructure
     */
	public void setTreeStructure(String s[])
	{
	    if (s == null || s.length == 0) { clear(); return;  }

	    rootNode = selectedNode = null;

		try	{ parseTreeStructure(s); }
		catch(Exception e) { System.err.println(e); }

		triggerRedraw();

	    invalidate();
	}

    /**
     * Gets a string array that reflects the current DTreeView's contents.
     * There is one string for each node in the array. That string
     * contains the text of the node indented with same number of
     * leading spaces as the depth of that node in the tree.
     * @return the string array that reflects the DTreeView's contents
     * @see #setTreeStructure
     */
	public String[] getTreeStructure()
	{
		//Create a vector representing current tree structure
		if (rootNode==null) return null;
		Vector nodesVector = new Vector(count);
		rootNode.depth = 0;
		vectorize(rootNode,false,nodesVector);

		//Convert this to a String[]
		int numNodes = nodesVector.size();
		String[] treeStructure = new String[numNodes];

		for (int i = 0; i < numNodes; i++)
		{
			DTreeNode thisNode = (DTreeNode)nodesVector.elementAt(i);

			//Add appropriate number of blanks
			String treeString = "";
			for (int numBlanks = 0;numBlanks < thisNode.depth;numBlanks++)
				treeString += ' ';

			//Add tree
			treeString += thisNode.text;

			//Put string into array
			treeStructure[i] = treeString;
		}

	    return treeStructure;
	}


    /**
       Parser
       Local
    */
	protected void parseTreeStructure(String tempStructure[]) throws Exception
	{
		for(int i = 0; i < tempStructure.length; i++)
		{
			String entry = tempStructure[i];
			int indentLevel = findLastPreSpace(entry)/*+1*/;

			if (indentLevel == -1)
				throw new Exception("parseTreeStructure: Empty strings in structure.");

			DTreeNode node = newTreeNode(entry.trim(), this);
			node.setDepth(indentLevel);

			if (rootNode == null)
			{
				if (indentLevel != 0)
					throw new Exception("parseTreeStructure: Indented nodes with no root level node.");

				append(node);
			}
			else
			{
				DTreeNode currentNode = rootNode;
                while(currentNode.sibling != null)
    				currentNode = currentNode.sibling;

				for(int j = 1; j < indentLevel; j++)
				{
					int numberOfChildren = currentNode.numberOfChildren;
					DTreeNode tempNode = null;

					if (numberOfChildren > 0)
					{
						tempNode = currentNode.child;

						while(tempNode.sibling != null)
							tempNode = tempNode.sibling;
					}

					if (tempNode != null)
						currentNode = tempNode;
					else
						break;
				}

				int diff = indentLevel - currentNode.getDepth();

				if (diff > 1)
					throw new Exception("parseTreeStructure: Generation gap. Node with no immediate parent.");

				if (diff == 1)
					insert(node, currentNode, CHILD);
				else
					insert(node, currentNode, NEXT);
			}
		}
	}

	/**
	 * Print out the text of each node in the DTreeView beginning with
	 * the given node.
	 * The nodes are printed out one per line with no indenting.
	 * @param node the first node to print
	 */
	public void printTree(DTreeNode node)
	{
		printTree(node, false);
	}

	/**
	 * Print out the text of each node in the DTreeView beginning with
	 * the given node.
	 * @param node the first node to print
	 * @param isIndented, if true, nodes will have hierarchical indenting,
	 * if false, all nodes will pe printed at the same level.
	 */
	public void printTree(DTreeNode node, boolean isIndented)
	{
	    if (node == null) return;

	    String padding = new String();
		if (isIndented)
		{
			for (int i = 0; i < node.depth; i++)
				padding = "  " + padding;
		}

	    System.out.println(padding + node.text);
	    printTree(node.child, isIndented);
	    printTree(node.sibling, isIndented);
	}


    /*
      Fonction locale
    */
	protected int findLastPreSpace(String s)
	{
		if(s != null && s.length() > 0)
		{
		    int length;

		    length = s.length();

		    if(s.charAt(0) != ' ' && s.charAt(0) != '\t')
		    {
		        return 0;
		    }

		    for(int i = 1; i < length; i++)
		    {
		        if(s.charAt(i) != ' ' && s.charAt(i) != '\t')
		        {
		            return i;
		        }
		    }
		}
	    return -1;
	}


}
